/*
 * Copyright 2005 Sun Microsystems, Inc.  All rights reserved.
 * PROPRIETARY/CONFIDENTIAL.  Use of this product is subject to license terms.
 */

package com.sun.portal.search.db;

import com.sun.portal.search.rdm.*;
import com.sun.portal.search.soif.*;
import com.sun.portal.search.util.*;

import com.google.soap.search.*;

import java.util.*;
import java.io.*;


/**
 *
 * Search engine interface implementation using Google search API
 *
 */
public class GoogleDb implements RDMDb {
  
  /** Google search engine API authorization key */
  private String clientKey;

  /** http proxy host for Google search SOAP connection */
  private String proxyHost = "";

  /** http proxy port for Google search SOAP connection */
  private String proxyPort = "";


  /**
   * Open a database
   * @param st Token for search
   * @param rootdir Database home dir
   * @param dbname Name of database from root database
   * @param rw RDMDb.WRITER or RDMDb.WRCREAT or RDMDb.READER
   * @param mode Unix mode
   * @throws RDMException
   */
  public void open(SToken st, String rootdir, String dbname, int rw, int mode) throws RDMException {
    SOIF dbsoif = DbManager.getRootDbEntry(st, dbname);
    clientKey = dbsoif.getValue("clientKey");
    proxyHost = dbsoif.getValue("proxyHost");
    proxyPort = dbsoif.getValue("proxyPort");
  }
    
    
  /**
   * Close database and index extents
   * @param st Token for search
   * @throws RDMException
   */
  public void close(SToken st) throws RDMException {
    ;//do nothing
  }


  /**
   * Empty the database
   * @param st Token for search
   * @param t RDM transaction handle
   * @throws RDMException
   */
  public int purge(SToken st, RDMTransaction t) throws RDMException {
    throw new RDMException("GoogleDb.purge() not implemented");
  }

  
  /**
   * Obtain the name for the search engine interface
   * @return the name of the search engine activity
   */
  public String getName() {
    return "Google Search";
  }
    

  /**
   * Submit the query to the search engine
   * @param st Token for search
   * @param query Search query
   * @param numHits Number of hits
   * @param view Set of attributes to be retrieved
   * @param sortOrder Order for sorting the results
   * @param t RDM transaction handle
   * @return RDMResultSet
   * @throws RDMException  
   */
  public RDMResultSet search(SToken st, String query, int numHits, Set view, String sortOrder, RDMTransaction t) throws RDMException {

    //Google search result set
    GoogleSearchResult gsr;

    try {
      gsr = googleSearch(clientKey, "search", query);
    }
    catch (GoogleSearchFault f) {
      throw new RDMException(f);
    }

    GoogleResultSet rs = new GoogleResultSet(st, this, query, t);
    rs.setGoogleSearchResult(gsr);
    rs.setView(view);	
    return rs;
  }
    
    
  /**
   * Call the Google APIs service to search Google
   * @param clientKey Google API authorization key
   * @param directive Search method
   * @param directiveArg Search query
   * @return GoogleSearchResult
   * @throws GoogleSearchFault
   */
  GoogleSearchResult googleSearch(String clientKey, String directive, String directiveArg) throws GoogleSearchFault {
	
    // Create a Google Search object, set the authorization key
    GoogleSearch s = new GoogleSearch();
    s.setKey(clientKey);

    if (proxyHost != null && !proxyHost.equals("") && proxyHost.indexOf(" ") == -1) {
      s.setProxyHost(proxyHost);

      if (proxyPort != null && !proxyPort.equals("") && proxyPort.indexOf(" ") == -1) {
	s.setProxyPort(Integer.parseInt(proxyPort));
      }
    }
    
    if (directive.equalsIgnoreCase("search")) {
      s.setQueryString(directiveArg);
      GoogleSearchResult gsr = s.doSearch();
      return gsr;
    } 
    else {
      throw new GoogleSearchFault("Invalid directive");
    }
  }
    
    
  /**
   * Number of documents in the database/index
   * @param st Token for search
   * @param t RDM transaction handle
   * @return search count
   * @throws RDMException
   */
  public int count(SToken st, RDMTransaction t) throws RDMException {
    return 2000000000;	
  }


  /**
   * Delete RD from database 
   * @param st Token for search
   * @param s SOIF document
   * @param view Set of attributes
   * @param flags Options
   * @param t RDM transaction handle
   * @throws RDMException
   */
  public void delete(SToken st, SOIF s, Set view, int flags, RDMTransaction t) throws RDMException {
    throw new RDMException("GoogleDb.delete() not implemented");
  }

    
  /**
   * Store - creates a transaction for atomic indexing, if none supplied.
   * @param st Token for search
   * @param soif SOIF document
   * @param view Set of Attributes
   * @param flags Options
   * @param t RDM transaction
   * @throws RDMException   
   */
  public void store(SToken st, SOIF soif, Set view, int flags, RDMTransaction t) throws RDMException {
    throw new RDMException("GoogleDb.store() not implemented");
  }
    

  /**
   * Update an RD with attributes set in View
   * @param st Token for search
   * @param insoif SOIF document
   * @param view Set of Attributes
   * @param flags Options
   * @param t RDM transaction
   * @throws RDMException   
   */
  public void update(SToken st, SOIF insoif, Set view, int flags, RDMTransaction t) throws RDMException {
    throw new RDMException("GoogleDb.update() not implemented");
  }


  /**
   * Retrieve RD from database, filtered by view
   * @param st Token for search
   * @param url URL string
   * @param view Set of Attributes
   * @param flags Options
   * @param t RDM transaction
   * @throws RDMException   
   */
  public SOIF fetch(SToken st, String url, Set view, int flags, RDMTransaction t) throws RDMException {
    throw new RDMException("GoogleDb.fetch() not implemented");
  }

    
  /**
   * Optimize the db
   * @param st Token for search
   * @throws RDMException   
   */
  public void optimize(SToken st) throws RDMException {
    throw new RDMException("GoogleDb.optimize() not implemented");
  }
    

  /**
   * Batch Index
   * @param st Token for search
   * @throws RDMException   
   */
   public void indexBatch(SToken st) throws RDMException {
     throw new RDMException("GoogleDb.indexBatch() not implemented");
   }


  /**
   * Recover the db - must be run stand alone (ie, no one else has the db open)
   * @param st Token for search
   * @param dbhome Database home dir
   * @param fatal Option
   * @throws RDMException   
   */
  public void recover(SToken st, String dbhome, boolean fatal) throws RDMException {
     throw new RDMException("GoogleDb.recover() not implemented");
  }
    

}
