/*
 * Copyright 2005 Sun Microsystems, Inc.  All rights reserved.
 * PROPRIETARY/CONFIDENTIAL.  Use of this product is subject to license terms.
 */

package com.sun.portal.search.db;

import com.sun.portal.search.rdm.*;
import com.sun.portal.search.soif.*;
import com.sun.portal.search.util.*;

import com.google.soap.search.*;

import java.util.*;


/**
 *
 * Google search result set implementation using RDMResultSet
 *
 */
public class GoogleResultSet extends RDMResultSet {
    
    
    /**
     * Creates a new instance of GoogleResultSet
     */
    GoogleResultSet(SToken st, RDMDb database, String query, RDMTransaction t) {
	super(st, database, query, t);
    }
    
    
    /**
     * Prepare Google Search Results
     */
    void setGoogleSearchResult(GoogleSearchResult gsr) {
	this.gsr = gsr;
	startHit = gsr.getStartIndex();
	endHit = gsr.getEndIndex();
	hitCount = gsr.getEstimatedTotalResultsCount();
	resultCount = endHit - startHit + 1;
	docCount = 2000000000;
    }
    
    /**
     * Add view to RDMResultSet
     */
    void setView(Set view) {
	this.view = view;
    }
    
    
    /**
     * Default view attribute set
     */
    static Set defaultView = new HashSet();
    static {
	defaultView.add("title");
	defaultView.add("description");
	defaultView.add("content-length");
    }
    
    
    static private void configureHighlighting() {
	
	// set up default highlighting from search config file
	String p = SearchConfig.getValue(SearchConfig.HIGHLIGHTS);
	if (p != null)
	    highlightsEnabled = p.equalsIgnoreCase("true");
	
	p = SearchConfig.getValue(SearchConfig.HIGHLIGHT_TAGS);
	if (p != null) {
	    String[] tags = String2Array.string2Array(p, ',');
	    if (tags != null && tags.length == 6)
		defaultHighlightTags = tags;
	}
	
	p = SearchConfig.getValue(SearchConfig.HIGHLIGHT_TAGS_FIELD);
	if (p != null) {
	    String[] tags = String2Array.string2Array(p, ',');
	    if (tags != null && tags.length == 6)
		defaultFldHighlightTags = tags;
	}
	
    }
    
    
      private void hlinit(String[] hltags) {
	
	if (!hlconfigured) {
	    synchronized (this.getClass()) {
		configureHighlighting();
		hlconfigured = true;
	    }
	}

	if (hlinited)
	    return;
	
	highlightTags = (String[])defaultHighlightTags.clone();
	fldHighlightTags = (String[])defaultFldHighlightTags.clone();

	if (hltags != null && hltags.length >= 6) System.arraycopy(hltags, 0, highlightTags, 0, 6);
	if (hltags != null && hltags.length == 12) System.arraycopy(hltags, 6, fldHighlightTags, 0, 6);
	hlinited = true;
      }  
    
    
    /**
     * Retrieve Search Results
     * @param i Document index
     * @param view Attributes
     * @param hltags Highlight tags
     * @return a SOIF hit, highlighted and filtered by view
     */
    public SOIF getResult(int i, Set view, String[] hltags) {
	if (hits == null) {
	    hits = gsr.getResultElements();
	}

	if (i >= hits.length) { 
	    return null;
	    //return null when the document index requested is
	    //greater than the google result count
	    //A temporary solution to avoid ArrayIndexOutOfBound 
	    //for Google search returns only 10 documents per search. 
	}

	GoogleSearchResultElement r = hits[i];
	
	hlinit(hltags);	
	
	SOIF s = new SOIF("DOCUMENT", r.getURL());
	
	// insert/map the attributes
	if (view == null)
	    view = this.view;
	if (view == null)
	    view = defaultView;
	if (view.contains("hl-description"))
	    s.insert("hl-description", convertHighlight(r.getSnippet(), true));
	if (view.contains("description"))
	    s.insert("description", convertHighlight(r.getSnippet(), false));
	if (view.contains("hl-title"))
	    s.insert("hl-title", convertFieldHighlight(r.getTitle(), true));
	if (view.contains("title"))
	    s.insert("title", convertFieldHighlight(r.getTitle(), false));
	if (view.contains("hl-url"))
	    s.insert("hl-url", r.getURL()); 
	if (view.contains("content-length"))
	    s.insert("content-length", r.getCachedSize());
	if (view.contains("score"))
	    s.insert("score", "100");
	return s;
	
    }
    
    
    /**
     * Get search hit count
     * @return total number of hits for this search
     */
    public long getHitCount() {
	return hitCount;
    }
    
    
    /**
     * Get document count
     * @return number of docs searched
     */
    public long getDocCount() {
	return docCount;
    }
    
    
    /**
     * Transform the result set to string
     * @return tring representation of this result set
     */
    public java.lang.String toString() {
	return super.toString();
    }
    
    
    /**
     * Get search result count
     * @return current number of hits in this set
     */
    public int getResultCount() {
	return resultCount;
    }
    
    
    /**
     * Convert google highlight to portal style highlight
     * @param s string with highlights to be converted
     * @param hl true convert highlights, false remove highlights
     * @return the highlight converted string
     */
    public String convertHighlight(String s, boolean hl) {
	if (highlightsEnabled && hl) {
	    // convert google highlights to RDM highlights
	    // treat each google hit as a passage - could do better...
	    String t1 = s.replaceAll("(?i)<B>\\.\\.\\.</B>", "...");
	    String t2 = t1.replaceAll("(?i)<B>", highlightTags[2] + highlightTags[4]);
	    String t3 = t2.replaceAll("(?i)</B>", highlightTags[5] + highlightTags[3]);
	    String t4 = t3.replaceAll("(?i)<BR>", "");
	    return t4;
	}
	else {
	    // strip google highlights
	    String t1 = s.replaceAll("(?i)<B>", "");
	    String t2 = t1.replaceAll("(?i)</B>", "");
	    String t3 = t2.replaceAll("(?i)<BR>", "");
	    return t3;
	}
    }
    
    /**
     * Convert google highlight to portal style field highlight
     * @param s string with highlights to be converted
     * @param hl true convert highlights, false remove highlights
     * @return the highlight converted string
     */
    public String convertFieldHighlight(String s, boolean hl) {
	if (highlightsEnabled && hl) {
	    // convert google highlights to RDM highlights
	    String t1 = s.replaceAll("(?i)<B>\\.\\.\\.</B>", "...");
	    String t2 = t1.replaceAll("(?i)<B>", fldHighlightTags[4]);
	    String t3 = t2.replaceAll("(?i)</B>", fldHighlightTags[5]);
	    return t3;
	}
	else {
	    // strip google highlights
	    String t1 = s.replaceAll("(?i)<B>", "");
	    String t2 = t1.replaceAll("(?i)</B>", "");
	    return t2;
	}
    }

    
    private GoogleSearchResult gsr;
    private GoogleSearchResultElement[] hits;
    int startHit;
    int endHit;
    long hitCount;
    long docCount;
    int resultCount;
    Set view;
    boolean hlinited = false;
    String[] highlightTags;
    String[] fldHighlightTags;

    // highlighting configuration
    static boolean hlconfigured = false;
    static boolean highlightsEnabled = true;
    static String[] defaultHighlightTags = { "... ", " ...", "<font color=bb0033>", "</font>", "<b>", "</b>" };
    static String[] defaultFldHighlightTags = { "", "", "", "", "<b>", "</b>" };    

}

