/*
 * Copyright 2005 Sun Microsystems, Inc.  All rights reserved.
 * PROPRIETARY/CONFIDENTIAL.  Use of this product is subject to license terms.
 */

package com.sun.portal.search.db;

import com.sun.portal.search.rdm.*;
import com.sun.portal.search.soif.*;
import com.sun.portal.search.util.*;

import javax.naming.*;

import java.util.*;
import java.util.regex.*;
import java.io.*;
import java.text.MessageFormat;

import java.sql.*;
import javax.sql.*;


/**
 *
 * Search engine interface implementation using JDBC to search a relational db
 *
 */
public class JDBCDb implements RDMDb {

    java.sql.Connection conn = null;
    java.sql.Statement stmt = null;
    String database;
    String queryPattern;


    /**
     * Store - creates a transaction for atomic indexing, if none supplied.
     * @param st Token for search
     * @param insoif SOIF document
     * @param view Set of Attributes
     * @param flags Options
     * @param t RDM transaction
     * @throws RDMException
     */
    public void store(SToken st, SOIF insoif, Set view, int flags, RDMTransaction t) throws RDMException {
	throw new RDMException("JDBCDb.store(): not implemented");
    }


    /**
     * Purge the database
     * @param st Token for search
     * @param t RDM transaction handle
     * @throws RDMException
     */
    public int purge(SToken st, RDMTransaction t) throws RDMException {
	throw new RDMException("JDBCDb.purge(): not implemented");
    }


    /**
     * Open a database
     * @param st Token for search
     * @param rootdir Database home dir
     * @param dbname Name of database from root database
     * @param rw RDMDb.WRITER or RDMDb.WRCREAT or RDMDb.READER
     * @param mode Unix mode
     * @throws RDMException
     */
    public void open(SToken st, String rootdir, String dbname, int rw, int mode) throws RDMException {

	database = dbname;

	SOIF dbsoif = DbManager.getRootDbEntry(st, dbname);

	String driver = dbsoif.getValue("driver");
	String dburl = dbsoif.getValue("databaseurl");
	String dbusr = dbsoif.getValue("username");
	String dbpwd = dbsoif.getValue("password");
	queryPattern = dbsoif.getValue("querypattern");

	try {
	    //Load JDBC Driver
	    Class.forName(driver).newInstance();
	    //Getting connection...
	    conn = DriverManager.getConnection(dburl, dbusr, dbpwd);
	} catch (Exception e) {
	    throw new RDMException("JDBCDb.open(): Failed to open jdbc connection", e);
	}
    }


    /**
     * Close database and index extents
     * @param st Token for search
     * @throws RDMException
     */
    public void close(SToken st) throws RDMException {
	try {
	    conn.close();
	} catch (Exception e) {
	    throw new RDMException("JDBCDb.close(): Error during close()", e);
	}
    }


    /**
     * Obtain the name for the search engine interface
     */
    public String getName() {
	return "JDBC Search";
    }


    /**
     * Submit the query to the search engine
     * @param st Token for search
     * @param queryString Search query
     * @param numHits Number of hits
     * @param view Set of attributes to be retrieved
     * @param sortOrder Order for sorting the results
     * @param t RDM transaction handle
     * @return RDMResultSet
     * @throws RDMException
     */
    public RDMResultSet search(SToken st, String queryString, int numHits, Set view, String sortOrder, RDMTransaction t) throws RDMException {

	ResultSet srs;
	DatabaseMetaData dmd;
	String query;

	if (isSQLQuery(queryString)) {
	    query = queryString;
	} else {
	    Object[] arguments = { queryString };
	    query = getSQLQuery(arguments, queryPattern);
	}

	try {
	    srs = jdbcSearch(query, numHits, view);
	    dmd = conn.getMetaData();
	}
	catch (Exception e) {
	    throw new RDMException(e);
	}

	JDBCResultSet js = new JDBCResultSet(st, this, query, t);
	js.setJDBCSearchResult(srs, dmd);
	return js;
    }


    /**
     * Search the relational db
     * @param query Search filter
     * @param numHits Number of search hits
     * @param view Set of Attributes
     * @return ResultSet
     * @throws RDMException
     */
    ResultSet jdbcSearch(String query, int numHits, Set view) throws Exception {
	ResultSet rs;

	try {
	    // Getting statement...
	    stmt = conn.createStatement();
	    // Executing query...
	    rs = stmt.executeQuery(query);
	}
	catch (Exception e) {
	    throw new RDMException(e);
	}    

	return rs;
    }


    /**
     * Transform the query format to SQL based on the argument pattern
     * @param arguments Query arguments
     * @param patterm Query pattern in SQL format
     * @return a query string that is in the SQL format defined by "pattern"
     */
    private String getSQLQuery(Object[] arguments, String pattern) {

	boolean singlequote = false;
	MessageFormat queryFormat;
	String query;

	if (pattern.indexOf("'") != -1) {
	    singlequote = true;
	}

	if (singlequote) {
	    queryFormat = new MessageFormat(pattern.replace('\'', '^'));
	    query = (queryFormat.format(arguments)).replace('^', '\'');
	} else {
	    queryFormat = new MessageFormat(pattern);
	    query = queryFormat.format(arguments);
	}

	return query;

    }


    /**
     * Check if the query is a SQL statement
     * @param query Search query
     * @return true or false
     */
    private boolean isSQLQuery(String qString) {

	String qs = qString.toLowerCase();

	if (qs.indexOf("select") != -1 &&
		qs.indexOf("select") < qs.indexOf("from"))
	    return true;
	else
	    return false;
    }


    /**
     * Get search count
     * @param st Token for search
     * @param t RDM transaction handle
     * @return Number of docs.
     */
    public int count(SToken st, RDMTransaction t) throws RDMException {
	return 250000000; 
    }


    /**
     * Delete RD from database
     * @param st Token for search
     * @param s SOIF document
     * @param view Set of attributes
     * @param flags Options
     * @param t RDM transaction handle
     * @throws RDMException
     */
    public void delete(SToken st, SOIF s, Set view, int flags, RDMTransaction t) throws RDMException {
	throw new RDMException("JDBCDb.delete() not implemented");
    }


    /**
     * Update an RD with attributes set in View
     * @param st Token for search
     * @param insoif SOIF document
     * @param view Set of Attributes
     * @param flags Options
     * @param t RDM transaction
     * @throws RDMException
     */
    public void update(SToken st, SOIF insoif, Set view, int flags, RDMTransaction t) throws RDMException {
	throw new RDMException("JDBCDb.update() not implemented");
    }


    /**
     * Retrieve RD from database, filtered by view
     * @param st Token for search
     * @param url URL string
     * @param view Set of Attributes
     * @param flags Options
     * @param t RDM transaction
     * @throws RDMException
     */
    public SOIF fetch(SToken st, String url, Set view, int flags, RDMTransaction t) throws RDMException {
	throw new RDMException("JDBCDb.fetch() not implemented");
    }


    /**
     * Optimize the db
     * @param st Token for search
     * @throws RDMException
     */
    public void optimize(SToken st) throws RDMException {
	throw new RDMException("JDBCDb.optimize() not implemented");
    }


    /**
     * Batch Index
     * @param st Token for search
     * @throws RDMException
     */
    public void indexBatch(SToken st) throws RDMException {
	throw new RDMException("JDBCDb.indexBatch() not implemented");
    }


    /**
     * Recover the db - must be run stand alone (ie, no one else has the db open)
     * @param st Token for search
     * @param dbhome Database home dir
     * @param fatal Option
     * @throws RDMException
     */
    public void recover(SToken st, String dbhome, boolean fatal) throws RDMException {
	throw new RDMException("JDBCDb.recover() not implemented");
    }



}
