/*
 * Copyright 2005 Sun Microsystems, Inc.  All rights reserved.
 * PROPRIETARY/CONFIDENTIAL.  Use of this product is subject to license terms.
 */

package com.sun.portal.search.db;

import com.sun.portal.search.rdm.*;
import com.sun.portal.search.soif.*;
import com.sun.portal.search.util.*;

import javax.naming.*;
import javax.naming.directory.*;

import java.util.*;
import java.io.*;
import java.text.MessageFormat;


/**
 *
 * Search engine interface implementation using JNDI to search a Directory db
 *
 */
public class JNDIDb implements RDMDb {

    Hashtable env = new Hashtable(11);    
    String queryPattern;


    /**
     * Store - creates a transaction for atomic indexing, if none supplied.
     * @param st Token for search
     * @param insoif SOIF document
     * @param view Set of Attributes
     * @param flags Options
     * @param t RDM transaction
     * @throws RDMException
     */
    public void store(SToken st, SOIF insoif, Set view, int flags, RDMTransaction t) throws RDMException {
	throw new RDMException("JNDIDb.store(): not implemented");
    }


    /**
     * Empty the database
     * @param st Token for search
     * @param t RDM transaction handle
     * @throws RDMException
     */
    public int purge(SToken st, RDMTransaction t) throws RDMException {
	throw new RDMException("JNDIDb.purge(): not implemented");
    }


    /**
     * Open a database
     * @param st Token for search
     * @param rootdir Database home dir
     * @param dbname Name of database from root database
     * @param rw RDMDb.WRITER or RDMDb.WRCREAT or RDMDb.READER
     * @param mode Unix mode
     * @throws RDMException
     */
    public void open(SToken st, String rootdir, String dbname, int rw, int mode) throws RDMException {

	SOIF dbsoif = DbManager.getRootDbEntry(st, dbname);

	String contextfactory = dbsoif.getValue("contextfactory");
	String providerurl = dbsoif.getValue("providerurl");
	String basedn = dbsoif.getValue("basedn");
	String binddn = dbsoif.getValue("binddn");
	String passwd = dbsoif.getValue("passwd");
	queryPattern = dbsoif.getValue("queryPattern");

	env.put(Context.INITIAL_CONTEXT_FACTORY, contextfactory);
	env.put(Context.PROVIDER_URL, providerurl + "/" + basedn );

	if ((binddn != null) && (binddn != "") && 
		(!binddn.equalsIgnoreCase("anonymous"))) {
	    env.put(Context.SECURITY_PRINCIPAL, binddn);
	    env.put(Context.SECURITY_CREDENTIALS, passwd);
	}

	/*
	 * we don't do this here since the connection will time out,
	 * for now we will reconnect with each request
	 */
	/*
	try {	    
	    // Create the initial directory context
	    dctx = new InitialDirContext(env);
	} catch (Exception e) {
	    throw new RDMException("JNDIDb.open(): Failed to open jndi context", e);
	}
	*/

    }


    /**
     * Close database and index extents
     * @param st Token for search
     * @throws RDMException
     */
    public void close(SToken st) throws RDMException {
	/*
	try {
	    dctx.close();
	} catch (Exception ne) {
	    throw new RDMException("JNDIDb.close(): Error during close()", ne);
	}
	*/
    }


    /**
     * Obtain the name for the search engine interface
     * @return the name of the search engine activity
     */
    public String getName() {
	return "JNDI Search";
    }


    /**
     * Submit the query to the search engine
     * @param st Token for search
     * @param queryString Search query
     * @param numHits Number of hits
     * @param view Set of attributes to be retrieved
     * @param sortOrder Order for sorting the results
     * @param t RDM transaction handle
     * @return RDMResultSet
     * @throws RDMException
     */
    public RDMResultSet search(SToken st, String queryString, int numHits, Set view, String sortOrder, RDMTransaction t) throws RDMException {

	numHits = Math.min(100, numHits);   
	NamingEnumeration jsr;
	DirContext ctx = null;
	String query=null;

	if (queryString.indexOf("=") != -1) {
	    query = queryString;
	} else {
	    Object[] arguments = { queryString };
	    query = getQueryFormat(arguments, queryPattern);
	}

	try {
	    //open and close cxn with each request
	    DirContext dctx = new InitialDirContext(env);
	    jsr = jndiSearch(query, numHits, view, dctx);
	    dctx.close();
	} catch (Exception e) {
	    throw new RDMException(e);
	}
	JNDIResultSet js = new JNDIResultSet(st, this, query, t);
	js.setJNDISearchResult(jsr);
	return js;
    }


    /**
     * Search the directory db
     * @param filter Search filter
     * @param numHits Number of search hits
     * @param view Set of Attributes
     * @param ctx Directory Context for search
     * @return NamingEnumeration
     * @throws RDMException
     */
    NamingEnumeration jndiSearch(String filter, int numHits, Set view, DirContext ctx) throws RDMException {

	NamingEnumeration ne;
	SearchControls sc = new SearchControls();
	sc.setCountLimit(numHits);
	sc.setSearchScope(SearchControls.SUBTREE_SCOPE);
	String[] attrs = null;
	if (view != null) {
	    attrs = (String[])view.toArray(new String[0]);
	    sc.setReturningAttributes(attrs);
	}
	try {
	    ne = ctx.search("", filter, sc);
	} catch (NamingException e) {
	    throw new RDMException(e);
	}
	return ne;
    }


    /**
     * Transform the query format based on the argument pattern
     * @param arguments Query arguments
     * @param patterm Query pattern in the directory search format
     * @return a query string that is in the format defined by "pattern" 
     */
    private String getQueryFormat(Object[] arguments, String pattern) {

	MessageFormat queryFormat = new MessageFormat("");
	queryFormat.applyPattern(pattern);
	return queryFormat.format(arguments);
    }


    /**
     * Get search count
     * @param st Token for search
     * @param t RDM transaction handle
     * @return Number of docs.
     */
    public int count(SToken st, RDMTransaction t) throws RDMException {
	return 250000000; // overflows int - need long
    }


    /**
     * Delete RD from database
     * @param st Token for search
     * @param s SOIF document
     * @param view Set of attributes
     * @param flags Options
     * @param t RDM transaction handle
     * @throws RDMException
     */
    public void delete(SToken st, SOIF s, Set view, int flags, RDMTransaction t) throws RDMException {
	throw new RDMException("JNDIDb.delete() not implemented");
    }


    /**
     * Update an RD with attributes set in View
     * @param st Token for search
     * @param insoif SOIF document
     * @param view Set of Attributes
     * @param flags Options
     * @param t RDM transaction
     * @throws RDMException
     */
    public void update(SToken st, SOIF insoif, Set view, int flags, RDMTransaction t) throws RDMException {
	throw new RDMException("JNDIDb.update() not implemented");
    }


    /**
     * Retrieve RD from database, filtered by view
     * @param st Token for search
     * @param url URL string
     * @param view Set of Attributes
     * @param flags Options
     * @param t RDM transaction
     * @throws RDMException
     */
    public SOIF fetch(SToken st, String url, Set view, int flags, RDMTransaction t) throws RDMException {
	throw new RDMException("JNDIDb.fetch() not implemented");
    }


    /**
     * Optimize the db
     * @param st Token for search
     * @throws RDMException
     */
    public void optimize(SToken st) throws RDMException {
	throw new RDMException("JNDIDb.optimize() not implemented");
    }


    /**
     * Batch Index
     * @param st Token for search
     * @throws RDMException
     */
    public void indexBatch(SToken st) throws RDMException {
	throw new RDMException("JNDIDb.indexBatch() not implemented");
    }


    /**
     * Recover the db - must be run stand alone (ie, no one else has the db open)
     * @param st Token for search
     * @param dbhome Database home dir
     * @param fatal Option
     * @throws RDMException
     */
    public void recover(SToken st, String dbhome, boolean fatal) throws RDMException {
	throw new RDMException("JNDIDb.recover() not implemented");
    }


}
